@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2021-2023 NXP
+# Copyright 2021-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -13,6 +13,7 @@ from typing import List
 
 import click
 
+from spsdk import SPSDK_DATA_FOLDER_COMMON
 from spsdk.apps.tp_utils import (
     TPHostConfig,
     device_help,
@@ -24,31 +25,31 @@ from spsdk.apps.tp_utils import (
     tp_device_options,
     tp_target_options,
 )
-from spsdk.apps.utils import (
-    INT,
-    CommandsTreeGroupAliasedGetCfgTemplate,
-    catch_spsdk_error,
+from spsdk.apps.utils import spsdk_logger
+from spsdk.apps.utils.common_cli_options import (
+    CommandsTreeGroup,
     spsdk_apps_common_options,
-    spsdk_logger,
+    spsdk_config_option,
+    spsdk_family_option,
+    spsdk_output_option,
 )
-from spsdk.apps.utils.utils import SPSDKAppError
-from spsdk.crypto import Encoding, ec
-from spsdk.crypto.loaders import extract_public_key, load_certificate_from_data
+from spsdk.apps.utils.utils import INT, SPSDKAppError, catch_spsdk_error
+from spsdk.crypto.certificate import Certificate
+from spsdk.crypto.hash import EnumHashAlgorithm
+from spsdk.crypto.keys import PublicKeyEcc
+from spsdk.crypto.types import SPSDKEncoding
+from spsdk.crypto.utils import extract_public_key
 from spsdk.exceptions import SPSDKError
-from spsdk.tp import TP_DATA_FOLDER, TpDevInterface, TpTargetInterface, TrustProvisioningHost
 from spsdk.tp.data_container import Container
 from spsdk.tp.data_container.payload_types import PayloadType
 from spsdk.tp.exceptions import SPSDKTpError
-from spsdk.tp.utils import (
-    get_supported_devices,
-    reconstruct_cryptography_key,
-    scan_tp_devices,
-    scan_tp_targets,
-)
+from spsdk.tp.tp_intf import TpDevInterface, TpTargetInterface
+from spsdk.tp.tphost import TrustProvisioningHost
+from spsdk.tp.utils import get_supported_devices, scan_tp_devices, scan_tp_targets
 from spsdk.utils.misc import load_binary, load_text, write_file
 
 
-@click.group(name="tphost", cls=CommandsTreeGroupAliasedGetCfgTemplate)
+@click.group(name="tphost", cls=CommandsTreeGroup)
 @spsdk_apps_common_options
 def main(log_level: int) -> int:
     """Application to secure Trust provisioning process of loading application in Un-trusted environment."""
@@ -59,12 +60,8 @@ def main(log_level: int) -> int:
 @main.command(name="load", no_args_is_help=True)
 @tp_device_options
 @tp_target_options
-@click.option(
-    "-f",
-    "--family",
-    type=click.Choice(get_supported_devices(), case_sensitive=False),
-    help="The target device name.",
-)
+@spsdk_family_option(families=get_supported_devices(), required=False)
+@spsdk_config_option(required=False)
 @click.option(
     "-fw",
     "--firmware",
@@ -83,12 +80,6 @@ def main(log_level: int) -> int:
     type=click.IntRange(0, 600, clamp=True),
     help="The target provisioning timeout in seconds.",
 )
-@click.option(
-    "-c",
-    "--config",
-    type=click.Path(exists=True, dir_okay=False),
-    help="Path to TPHost config file.",
-)
 @click.option(
     "-l",
     "--audit-log",
@@ -146,7 +137,7 @@ def load(
         scan_func=scan_tp_targets,
         print_func=click.echo,
     )
-    tp_target_instance = tp_interface.create_interface()
+    tp_target_instance = tp_interface.create_interface(family=tp_config.family)
     assert isinstance(tp_target_instance, TpTargetInterface)
 
     tp_worker = TrustProvisioningHost(tp_device_instance, tp_target_instance, click.echo)
@@ -162,12 +153,8 @@ def load(
 
 @main.command(name="load-tpfw", no_args_is_help=True)
 @tp_target_options
-@click.option(
-    "-f",
-    "--family",
-    type=click.Choice(get_supported_devices(), case_sensitive=False),
-    help="The target device name.",
-)
+@spsdk_family_option(families=get_supported_devices(), required=False)
+@spsdk_config_option(required=False)
 @click.option(
     "-pfw",
     "--prov-firmware",
@@ -180,12 +167,6 @@ def load(
     type=click.IntRange(0, 600, clamp=True),
     help="The target provisioning timeout in seconds.",
 )
-@click.option(
-    "-c",
-    "--config",
-    type=click.Path(exists=True, dir_okay=False),
-    help="Path to TPHost config file.",
-)
 @click.option(
     "-s",
     "--skip-test",
@@ -222,7 +203,7 @@ def load_tpfw(
         scan_func=scan_tp_targets,
         print_func=click.echo,
     )
-    tp_target_instance = tp_interface.create_interface()
+    tp_target_instance = tp_interface.create_interface(family=tp_config.family)
     assert isinstance(tp_target_instance, TpTargetInterface)
 
     tp_worker = TrustProvisioningHost(
@@ -241,27 +222,17 @@ def load_tpfw(
 
 
 @main.command(name="get-template", no_args_is_help=True)
-@click.option(
-    "-f",
-    "--family",
-    type=click.Choice(get_supported_devices(), case_sensitive=False),
-    default="lpc55s6x",
-    help="Chip family to generate the TPHost config for.",
-)
-@click.option(
-    "-o",
-    "--output",
-    type=click.Path(dir_okay=False, resolve_path=True),
-    required=True,
-    help="The output YAML template configuration file name.",
-)
+@spsdk_family_option(families=get_supported_devices())
+@spsdk_output_option(force=True)
 # pylint: disable=unused-argument   # preparation for the future
 def get_template(
     family: str,
     output: str,
 ) -> None:
     """Command to generate tphost template of configuration YML file."""
-    template = load_text(os.path.join(TP_DATA_FOLDER, "tphost_cfg_template.yaml"))
+    template_name = "tphost_cfg_template.yaml"
+    template = load_text(os.path.join(SPSDK_DATA_FOLDER_COMMON, "tp", template_name))
+    template = template.replace("TMP_FAMILY", family)
     write_file(template, output)
 
     click.echo(f"The configuration template created. {output}")
@@ -282,12 +253,6 @@ def get_template(
     required=True,
     help="Path to private/public key to verify the TP audit log yaml file.",
 )
-@click.option(
-    "-d",
-    "--destination",
-    type=click.Path(file_okay=False),
-    help="Destination directory for certificate extraction(non-existent or empty).",
-)
 @click.option(
     "-e",
     "--encoding",
@@ -297,14 +262,14 @@ def get_template(
     help="X509 certificate encoding.",
 )
 @click.option(
-    "-n",
+    "-sn",
     "--skip-nxp",
     is_flag=True,
     default=False,
     help="Skip extracting the NXP Devattest certificates.",
 )
 @click.option(
-    "-o",
+    "-so",
     "--skip-oem",
     is_flag=True,
     default=False,
@@ -327,38 +292,36 @@ def get_template(
     type=INT(),
     help=f"How many processes to use; if not specified use cpu_count: {os.cpu_count()}",
 )
-@click.option(
-    "--force-rewrite",
-    is_flag=True,
-    default=False,
-    help="Rewrite certificates in 'destination' directory.",
+@spsdk_output_option(
+    required=False,
+    directory=True,
+    force=True,
+    help="Destination directory for certificate extraction(non-existent or empty).",
 )
 def verify(
     audit_log: str,
     audit_log_key: str,
-    destination: str,
+    output: str,
     encoding: str,
     skip_nxp: bool,
     skip_oem: bool,
     cert_index: int,
     processes: int,
-    force_rewrite: bool,
 ) -> None:
     """Verify audit log integrity and optionally extract certificates.
 
-    Certificate extraction takes place if `-d/--destination` is specified.
+    Certificate extraction takes place if `-o/--output` is specified.
     """
     TrustProvisioningHost.verify_extract_log(
         audit_log=audit_log,
         audit_log_key=audit_log_key,
-        destination=destination,
+        destination=output,
         skip_nxp=skip_nxp,
         skip_oem=skip_oem,
         cert_index=cert_index,
-        encoding=Encoding.PEM if encoding.lower() == "pem" else Encoding.DER,
+        encoding=SPSDKEncoding.PEM if encoding.lower() == "pem" else SPSDKEncoding.DER,
         max_processes=processes,
         info_print=click.echo,
-        force_rewrite=force_rewrite,
     )
 
 
@@ -370,12 +333,8 @@ def verify(
     type=click.IntRange(0, 600, clamp=True),
     help="The target provisioning timeout in seconds.",
 )
-@click.option(
-    "-c",
-    "--config",
-    type=click.Path(exists=True, dir_okay=False),
+@spsdk_config_option(
     help="Path to configuration file (parameters on CLI take precedence).",
-    required=False,
 )
 @click.option(
     "-l",
@@ -421,11 +380,10 @@ def check_log_owner(
 
 @main.command(name="get-tp-response", no_args_is_help=True)
 @tp_target_options
-@click.option(
-    "-f",
-    "--family",
-    type=click.Choice(get_supported_devices(), case_sensitive=False),
-    help="The target device name.",
+@spsdk_family_option(families=get_supported_devices(), required=False)
+@spsdk_config_option(
+    help="Path to configuration file (parameters on CLI take precedence).",
+    required=False,
 )
 @click.option(
     "-to",
@@ -433,13 +391,6 @@ def check_log_owner(
     type=click.IntRange(0, 600, clamp=True),
     help="The target provisioning timeout in seconds.",
 )
-@click.option(
-    "-c",
-    "--config",
-    type=click.Path(exists=True, dir_okay=False),
-    help="Path to TPHost configuration file (parameters on CLI take precedence).",
-    required=False,
-)
 @click.option(
     "-r",
     "--response-file",
@@ -447,6 +398,20 @@ def check_log_owner(
     help="Path where to store the TP_RESPONSE",
     required=True,
 )
+@click.option(
+    "-k",
+    "--key-flags",
+    type=INT(),
+    default=0x01,
+    help="OEM Key Flags. Default: 0x01",
+)
+@click.option(
+    "-s",
+    "--save-debug-data",
+    is_flag=True,
+    default=False,
+    help="Save the data being transferred (for debugging purposes).",
+)
 def get_tp_response(
     tp_target: str,
     tp_target_parameter: List[str],
@@ -454,6 +419,8 @@ def get_tp_response(
     timeout: int,
     config: str,
     response_file: str,
+    key_flags: int,
+    save_debug_data: bool,
 ) -> None:
     """Retrieve TP_RESPONSE from the target."""
     TPHostConfig.SCHEMA_MEMBERS = ["family", "tp_timeout", "target"]
@@ -473,7 +440,7 @@ def get_tp_response(
         scan_func=scan_tp_targets,
         print_func=click.echo,
     )
-    tp_target_instance = tp_interface.create_interface()
+    tp_target_instance = tp_interface.create_interface(family=tp_config.family)
     assert isinstance(tp_target_instance, TpTargetInterface)
 
     tp_worker = TrustProvisioningHost(
@@ -484,6 +451,8 @@ def get_tp_response(
     tp_worker.get_tp_response(
         response_file=response_file,
         timeout=tp_config.timeout,
+        oem_key_flags=key_flags,
+        save_debug_data=save_debug_data,
     )
 
 
@@ -526,25 +495,24 @@ def check_cot(root_cert: str, intermediate_cert: str, tp_response: str) -> None:
 
     nxp_prod_cert_data = load_binary(intermediate_cert)
     try:
-        nxp_prod_cert = load_certificate_from_data(nxp_prod_cert_data)
-        nxp_prod_puk = nxp_prod_cert.public_key()
+        nxp_prod_cert = Certificate.parse(nxp_prod_cert_data)
+        nxp_prod_puk = nxp_prod_cert.get_public_key()
     except SPSDKError as e:
         logging.debug(str(e))
         if nxp_glob_puk:
             raise SPSDKAppError(f"Unable to load NXP_PROD certificate: {str(e)}") from e
         click.echo("Failed to load NXP_PROD as certificate, attempting to load raw public key")
-        nxp_prod_puk = reconstruct_cryptography_key(key_material=nxp_prod_cert_data)
-    assert isinstance(nxp_prod_puk, ec.EllipticCurvePublicKey)
+        nxp_prod_puk = PublicKeyEcc.parse(nxp_prod_cert_data)
 
     if nxp_glob_puk:
-        assert isinstance(nxp_glob_puk, ec.EllipticCurvePublicKey)
+        assert isinstance(nxp_glob_puk, PublicKeyEcc)
         message = "validating NXP_PROD_CERT signature..."
         try:
             assert nxp_prod_cert.signature_hash_algorithm
-            nxp_glob_puk.verify(
+            nxp_glob_puk.verify_signature(
                 nxp_prod_cert.signature,
                 nxp_prod_cert.tbs_certificate_bytes,
-                ec.ECDSA(nxp_prod_cert.signature_hash_algorithm),
+                EnumHashAlgorithm.from_label(nxp_prod_cert.signature_hash_algorithm.name),
             )
             message += "OK"
         except Exception:
@@ -576,7 +544,7 @@ def check_cot(root_cert: str, intermediate_cert: str, tp_response: str) -> None:
 
     assert nxp_die_cert
     message = "Validating NXP_DIE_ID_Devattest_CERT signature..."
-    if nxp_die_cert.validate(nxp_prod_puk):  # type: ignore
+    if nxp_die_cert.validate(nxp_prod_puk.export(SPSDKEncoding.DER)):
         message += "OK"
     else:
         message += "FAILED!"
@@ -588,8 +556,8 @@ def check_cot(root_cert: str, intermediate_cert: str, tp_response: str) -> None:
         nxp_die_puk_data = nxp_die_cert.get_entry(PayloadType.NXP_DIE_ATTEST_AUTH_PUK).payload
     except SPSDKTpError:
         nxp_die_puk_data = nxp_die_cert.get_entry(PayloadType.NXP_DIE_ID_AUTH_PUK).payload
-    nxp_die_puk = reconstruct_cryptography_key(nxp_die_puk_data)
-    if tp_response_container.validate(nxp_die_puk):  # type: ignore
+    nxp_die_puk = PublicKeyEcc.parse(nxp_die_puk_data)
+    if tp_response_container.validate(nxp_die_puk.export(SPSDKEncoding.DER)):
         message += "OK"
     else:
         message += "FAILED!"
