@@ -1,12 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Wrappers for various types of debuggers."""
 
-from .debug_probe import DebugProbe
 from .debug_probe_jlink import DebugProbePyLink
 from .debug_probe_pyocd import DebugProbePyOCD
+
+__all__ = ["DebugProbePyLink", "DebugProbePyOCD"]
