@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2023 NXP
+# Copyright 2020-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module for DebugMailbox PyLink Debug probes support."""
@@ -15,40 +15,39 @@ from pylink.errors import JLinkException
 from spsdk.utils.misc import value_to_int
 
 from .debug_probe import (
-    DebugProbe,
+    DebugProbeLocal,
+    DebugProbes,
+    ProbeDescription,
     SPSDKDebugProbeError,
     SPSDKDebugProbeNotOpenError,
     SPSDKDebugProbeTransferError,
 )
 
 logger = logging.getLogger(__name__)
-JLINK_LOGGER = logger.getChild("PyLink")
-JLINK_LOGGER.setLevel(logging.ERROR)
+logger_pylink = logger.getChild("PyLink")
+logger_pylink.setLevel(logging.CRITICAL)
 
 
-def set_logger(level: int) -> None:
-    """Sets the log level for this module.
-
-    param level: Requested level.
-    """
-    logger.setLevel(level)
-
-
-set_logger(logging.ERROR)
-
-
-class DebugProbePyLink(DebugProbe):
+class DebugProbePyLink(DebugProbeLocal):
     """Class to define PyLink package interface for NXP SPSDK."""
 
+    NAME = "jlink"
+
     @staticmethod
     def get_options_help() -> Dict[str, str]:
         """Get full list of options of debug probe.
 
         :return: Dictionary with individual options. Key is parameter name and value the help text.
         """
-        return {
-            "frequency": "Set the communication frequency in KHz, default is 100KHz",
-        }
+        options_help = {}
+        options_help.update(
+            {
+                "frequency": "Set the communication frequency in KHz, default is 100KHz",
+            }
+        )
+        options_help.update(DebugProbeLocal.get_options_help())
+
+        return options_help
 
     @classmethod
     def get_jlink_lib(cls) -> pylink.JLink:
@@ -59,10 +58,10 @@ class DebugProbePyLink(DebugProbe):
         """
         try:
             return pylink.JLink(
-                log=JLINK_LOGGER.info,
-                detailed_log=JLINK_LOGGER.debug,
-                error=JLINK_LOGGER.error,
-                warn=JLINK_LOGGER.warn,
+                log=logger_pylink.info,
+                detailed_log=logger_pylink.debug,
+                error=logger_pylink.error,
+                warn=logger_pylink.warn,
             )
         except TypeError as exc:
             raise SPSDKDebugProbeError("Cannot open Jlink DLL") from exc
@@ -73,7 +72,6 @@ class DebugProbePyLink(DebugProbe):
         The PyLink initialization function for SPSDK library to support various DEBUG PROBES.
         """
         super().__init__(hardware_id, options)
-        set_logger(logging.root.level)
         self.pylink = None
 
         logger.debug("The SPSDK PyLink Interface has been initialized")
@@ -81,7 +79,7 @@ class DebugProbePyLink(DebugProbe):
     @classmethod
     def get_connected_probes(
         cls, hardware_id: Optional[str] = None, options: Optional[Dict] = None
-    ) -> list:
+    ) -> DebugProbes:
         """Get all connected probes over PyLink.
 
         This functions returns the list of all connected probes in system by PyLink package.
@@ -91,9 +89,6 @@ class DebugProbePyLink(DebugProbe):
         :param options: The options dictionary
         :return: probe_description
         """
-        # pylint: disable=import-outside-toplevel
-        from .utils import DebugProbes, ProbeDescription
-
         jlink = DebugProbePyLink.get_jlink_lib()
 
         probes = DebugProbes()
@@ -104,7 +99,7 @@ class DebugProbePyLink(DebugProbe):
                     ProbeDescription(
                         "Jlink",
                         str(probe.SerialNumber),
-                        f"Segger {probe.acProduct.decode('utf-8')}: {str(probe.SerialNumber)}",
+                        f"Segger {probe.acProduct.decode('utf-8')}",
                         DebugProbePyLink,
                     )
                 )
@@ -144,9 +139,6 @@ class DebugProbePyLink(DebugProbe):
             self.pylink.coresight_configure()
             self.pylink.set_speed(speed=value_to_int(self.options.get("frequency", 100)))
             # Power Up the system and debug and clear sticky errors
-            if self.options.get("use_jtag") is not None:
-                # Currently clear_sticky_errors has been defined only for SWD (uncleared for JTAG-DP)
-                self.coresight_reg_write(access_port=False, addr=4, data=0x50000F20)
             self.clear_sticky_errors()
             self.power_up_target()
 
@@ -242,7 +234,4 @@ class DebugProbePyLink(DebugProbe):
         if not self.disable_reinit:
             assert self.pylink
             self.pylink.coresight_configure()
-            if self.options.get("use_jtag") is not None:
-                # Currently clear_sticky_errors has been defined only for SWD (uncleared for JTAG-DP)
-                self.coresight_reg_write(access_port=False, addr=4, data=0x50000F20)
             self._reinit_target()
