@@ -2,13 +2,13 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2016-2018 Martin Olejar
-# Copyright 2019-2023 NXP
+# Copyright 2019-2024 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Status and error codes used by the MBoot protocol."""
 
-from spsdk.utils.easy_enum import Enum
+from spsdk.utils.spsdk_enum import SpsdkEnum
 
 ########################################################################################################################
 # McuBoot Status Codes (Errors)
@@ -16,7 +16,7 @@ from spsdk.utils.easy_enum import Enum
 
 # pylint: disable=line-too-long
 # fmt: off
-class StatusCode(Enum):
+class StatusCode(SpsdkEnum):
     """McuBoot status codes."""
 
     SUCCESS                 = (0, "Success", "Success")
@@ -28,27 +28,39 @@ class StatusCode(Enum):
     NO_TRANSFER_IN_PROGRESS = (6, "NoTransferInProgress", "No Transfer In Progress Error")
 
     # Flash driver errors.
-    FLASH_SIZE_ERROR                = (100, "FlashSizeError", "FLASH Driver: Size Error")
-    FLASH_ALIGNMENT_ERROR           = (101, "FlashAlignmentError", "FLASH Driver: Alignment Error")
-    FLASH_ADDRESS_ERROR             = (102, "FlashAddressError", "FLASH Driver: Address Error")
-    FLASH_ACCESS_ERROR              = (103, "FlashAccessError", "FLASH Driver: Access Error")
-    FLASH_PROTECTION_VIOLATION      = (104, "FlashProtectionViolation", "FLASH Driver: Protection Violation")
-    FLASH_COMMAND_FAILURE           = (105, "FlashCommandFailure", "FLASH Driver: Command Failure")
-    FLASH_UNKNOWN_PROPERTY          = (106, "FlashUnknownProperty", "FLASH Driver: Unknown Property")
-    FLASH_REGION_EXECUTE_ONLY       = (108, "FlashRegionExecuteOnly", "FLASH Driver: Region Execute Only")
-    FLASH_EXEC_IN_RAM_NOT_READY     = (109, "FlashExecuteInRamFunctionNotReady", "FLASH Driver: Execute In RAM Function Not Ready")
-    FLASH_COMMAND_NOT_SUPPORTED     = (111, "FlashCommandNotSupported", "FLASH Driver: Command Not Supported")
-    FLASH_ECC_ERROR                 = (116, "FlashEccError", "FLASH Driver: ECC Error")
-    FLASH_OUT_OF_DATE_CFPA_PAGE     = (132, "FlashOutOfDateCfpaPage", "FLASH Driver: Out Of Date CFPA Page")
-    FLASH_BLANK_IFR_PAGE_DATA       = (133, "FlashBlankIfrPageData", "FLASH Driver: Blank IFR Page Data")
+    FLASH_SIZE_ERROR                  = (100, "FlashSizeError", "FLASH Driver: Size Error")
+    FLASH_ALIGNMENT_ERROR             = (101, "FlashAlignmentError", "FLASH Driver: Alignment Error")
+    FLASH_ADDRESS_ERROR               = (102, "FlashAddressError", "FLASH Driver: Address Error")
+    FLASH_ACCESS_ERROR                = (103, "FlashAccessError", "FLASH Driver: Access Error")
+    FLASH_PROTECTION_VIOLATION        = (104, "FlashProtectionViolation", "FLASH Driver: Protection Violation")
+    FLASH_COMMAND_FAILURE             = (105, "FlashCommandFailure", "FLASH Driver: Command Failure")
+    FLASH_UNKNOWN_PROPERTY            = (106, "FlashUnknownProperty", "FLASH Driver: Unknown Property")
+    FLASH_ERASE_KEY_ERROR             = (107, "FlashEraseKeyError", "FLASH Driver: Provided Key Does Not Match Programmed Flash Memory Key")
+    FLASH_REGION_EXECUTE_ONLY         = (108, "FlashRegionExecuteOnly", "FLASH Driver: Region Execute Only")
+    FLASH_EXEC_IN_RAM_NOT_READY       = (109, "FlashExecuteInRamFunctionNotReady", "FLASH Driver: Execute In RAM Function Not Ready")
+    FLASH_COMMAND_NOT_SUPPORTED       = (111, "FlashCommandNotSupported", "FLASH Driver: Command Not Supported")
+    FLASH_READ_ONLY_PROPERTY          = (112, "FlashReadOnlyProperty", "FLASH Driver: Flash Memory Property Is Read-Only")
+    FLASH_INVALID_PROPERTY_VALUE      = (113, "FlashInvalidPropertyValue", "FLASH Driver: Flash Memory Property Value Out Of Range")
+    FLASH_INVALID_SPECULATION_OPTION  = (114, "FlashInvalidSpeculationOption", "FLASH Driver: Flash Memory Prefetch Speculation Option Is Invalid")
+    FLASH_ECC_ERROR                   = (116, "FlashEccError", "FLASH Driver: ECC Error")
+    FLASH_COMPARE_ERROR               = (117, "FlashCompareError", "FLASH Driver: Destination And Source Memory Contents Do Not Match")
+    FLASH_REGULATION_LOSS             = (118, "FlashRegulationLoss", "FLASH Driver: Loss Of Regulation During Read")
+    FLASH_INVALID_WAIT_STATE_CYCLES   = (119, "FlashInvalidWaitStateCycles", "FLASH Driver: Wait State Cycle Set To Read/Write Mode Is Invalid")
+    FLASH_OUT_OF_DATE_CFPA_PAGE       = (132, "FlashOutOfDateCfpaPage", "FLASH Driver: Out Of Date CFPA Page")
+    FLASH_BLANK_IFR_PAGE_DATA         = (133, "FlashBlankIfrPageData", "FLASH Driver: Blank IFR Page Data")
     FLASH_ENCRYPTED_REGIONS_ERASE_NOT_DONE_AT_ONCE = (134, "FlashEncryptedRegionsEraseNotDoneAtOnce", "FLASH Driver: Encrypted Regions Erase Not Done At Once")
     FLASH_PROGRAM_VERIFICATION_NOT_ALLOWED = (135, "FlashProgramVerificationNotAllowed", "FLASH Driver: Program Verification Not Allowed")
-    FLASH_HASH_CHECK_ERROR          = (136, "FlashHashCheckError", "FLASH Driver: Hash Check Error")
-    FLASH_SEALED_PFR_REGION         = (137, "FlashSealedPfrRegion", "FLASH Driver: Sealed PFR Region")
-    FLASH_PFR_REGION_WRITE_BROKEN   = (138, "FlashPfrRegionWriteBroken", "FLASH Driver: PFR Region Write Broken")
-    FLASH_NMPA_UPDATE_NOT_ALLOWED   = (139, "FlashNmpaUpdateNotAllowed", "FLASH Driver: NMPA Update Not Allowed")
+    FLASH_HASH_CHECK_ERROR            = (136, "FlashHashCheckError", "FLASH Driver: Hash Check Error")
+    FLASH_SEALED_PFR_REGION           = (137, "FlashSealedPfrRegion", "FLASH Driver: Sealed PFR Region")
+    FLASH_PFR_REGION_WRITE_BROKEN     = (138, "FlashPfrRegionWriteBroken", "FLASH Driver: PFR Region Write Broken")
+    FLASH_NMPA_UPDATE_NOT_ALLOWED     = (139, "FlashNmpaUpdateNotAllowed", "FLASH Driver: NMPA Update Not Allowed")
     FLASH_CMPA_CFG_DIRECT_ERASE_NOT_ALLOWED = (140, "FlashCmpaCfgDirectEraseNotAllowed", "FLASH Driver: CMPA Cfg Direct Erase Not Allowed")
-    FLASH_PFR_BANK_IS_LOCKED        = (141, "FlashPfrBankIsLocked", "FLASH Driver: PFR Bank Is Locked")
+    FLASH_PFR_BANK_IS_LOCKED          = (141, "FlashPfrBankIsLocked", "FLASH Driver: PFR Bank Is Locked")
+    FLASH_CFPA_SCRATCH_PAGE_INVALID   = (148, "FlashCfpaScratchPageInvalid", "FLASH Driver: CFPA Scratch Page Invalid")
+    FLASH_CFPA_VERSION_ROLLBACK_DISALLOWED = (149, "FlashCfpaVersionRollbackDisallowed", "FLASH Driver: CFPA Version Rollback Disallowed")
+    FLASH_READ_HIDING_AREA_DISALLOWED = (150, "FlashReadHidingAreaDisallowed", "FLASH Driver: Flash Memory Hiding Read Not Allowed")
+    FLASH_MODIFY_PROTECTED_AREA_DISALLOWED = (151, "FlashModifyProtectedAreaDisallowed", "FLASH Driver: Flash Firewall Page Locked Erase And Program Are Not Allowed")
+    FLASH_COMMAND_OPERATION_IN_PROGRESS = (152, "FlashCommandOperationInProgress", "FLASH Driver: Flash Memory State Busy Flash Memory Command Is In Progress")
 
     # I2C driver errors.
     I2C_SLAVE_TX_UNDERRUN       = (200, "I2cSlaveTxUnderrun", "I2C Driver: Slave Tx Underrun")
@@ -256,6 +268,16 @@ class StatusCode(Enum):
     SPIFINOR_COMMAND_FAILURE        = (22006, "SPIFINOR_CommandFailure", "SPIFINOR: Command Failure")
     SPIFINOR_SFDP_NOT_FOUND         = (22007, "SPIFINOR_SFDP_NotFound", "SPIFINOR: SFDP Not Found")
 
+    # EDGELOCK ENCLAVE statuses.
+    EDGELOCK_INVALID_RESPONSE       = (30000, "EDGELOCK_InvalidResponse", "EDGELOCK: Invalid Response")
+    EDGELOCK_RESPONSE_ERROR         = (30001, "EDGELOCK_ResponseError", "EDGELOCK: Response Error")
+    EDGELOCK_ABORT                  = (30002, "EDGELOCK_Abort", "EDGELOCK: Abort")
+    EDGELOCK_OPERATION_FAILED       = (30003, "EDGELOCK_OperationFailed", "EDGELOCK: Operation Failed")
+    EDGELOCK_OTP_PROGRAM_FAILURE    = (30004, "EDGELOCK_OTPProgramFailure", "EDGELOCK: OTP Program Failure")
+    EDGELOCK_OTP_LOCKED             = (30005, "EDGELOCK_OTPLocked", "EDGELOCK: OTP Locked")
+    EDGELOCK_OTP_INVALID_IDX        = (30006, "EDGELOCK_OTPInvalidIDX", "EDGELOCK: OTP Invalid IDX")
+    EDGELOCK_INVALID_LIFECYCLE      = (30007, "EDGELOCK_InvalidLifecycle", "EDGELOCK: Invalid Lifecycle")
+
     # OTP statuses.
     OTP_INVALID_ADDRESS             = (52801, "OTP_InvalidAddress", "OTD: Invalid OTP address")
     OTP_PROGRAM_FAIL                = (52802, "OTP_ProgrammingFail", "OTD: Programming failed")
@@ -273,7 +295,7 @@ class StatusCode(Enum):
     # TrustProvisioning statuses.
     TP_SUCCESS                  = (0,     "TP_SUCCESS", "TP: SUCCESS")
     TP_GENERAL_ERROR            = (80000, "TP_GENERAL_ERROR", "TP: General error")
-    TP_MBEDTLS_ERROR            = (80001, "TP_MBEDTLS_ERROR", "TP: mBED TLS error")
+    TP_CRYPTO_ERROR             = (80001, "TP_CRYPTO_ERROR", "TP: Error during cryptographic operation")
     TP_NULLPTR_ERROR            = (80002, "TP_NULLPTR_ERROR", "TP: NULL pointer dereference or when buffer could not be allocated")
     TP_ALREADYINITIALIZED       = (80003, "TP_ALREADYINITIALIZED", "TP: Already initialized")
     TP_BUFFERSMALL              = (80004, "TP_BUFFERSMALL", "TP: Buffer is too small")
@@ -301,13 +323,24 @@ class StatusCode(Enum):
     TP_PACKET_ERROR             = (80026, "TP_PACKET_ERROR", "TP: Error during packet sending/receiving")
     TP_PACKET_DATA_ERROR        = (80027, "TP_PACKET_DATA_ERROR", "TP: Data in packet handle are invalid")
     TP_UNKNOWN_COMMAND          = (80028, "TP_UNKNOWN_COMMAND", "TP: Unknown command was received")
+    TP_SB3_FILE_ERROR           = (80029, "TP_SB3_FILE_ERROR", "TP: Error during processing SB3 file")
     # TP_CRITICAL_ERROR_START     (80100)
     TP_GENERAL_CRITICAL_ERROR       = (80101, "TP_GENERAL_CRITICAL_ERROR", "TP: Critical error")
-    TP_MBEDTLS_CRITICAL_ERROR       = (80102, "TP_MBEDTLS_CRITICAL_ERROR", "TP: Initialization or seed generation in MbedTLS failed")
+    TP_CRYPTO_CRITICAL_ERROR        = (80102, "TP_CRYPTO_CRITICAL_ERROR", "TP: Error of crypto module which prevents proper functionality")
     TP_PUF_CRITICAL_ERROR           = (80103, "TP_PUF_CRITICAL_ERROR", "TP: Initialization or start of the PUF periphery failed")
     TP_PFR_CRITICAL_ERROR           = (80104, "TP_PFR_CRITICAL_ERROR", "TP: Initialization of PFR or reading of activation code failed")
     TP_PERIPHERAL_CRITICAL_ERROR    = (80105, "TP_PERIPHERAL_CRITICAL_ERROR", "TP: Peripheral failure")
+    TP_PRINCE_CRITICAL_ERROR        = (80106, "TP_PRINCE_CRITICAL_ERROR", "TP: Error during PRINCE encryption/decryption")
+    TP_SHA_CHECK_CRITICAL_ERROR     = (80107, "TP_SHA_CHECK_CRITICAL_ERROR", "TP: SHA check verification failed")
 
+    # IAP statuses.
+    IAP_INVALID_ARGUMENT       = (100001, "IAP_InvalidArgument", "IAP: Invalid Argument Detected During API Execution")
+    IAP_OUT_OF_MEMORY          = (100002, "IAP_OutOfMemory", "IAP: Heap Size Not Large Enough During API Execution")
+    IAP_READ_DISALLOWED        = (100003, "IAP_ReadDisallowed ", "IAP: Read Memory Operation Disallowed During API Execution")
+    IAP_CUMULATIVE_WRITE       = (100004, "IAP_CumulativeWrite", "IAP: Flash Memory Region To Be Programmed Is Not Empty")
+    IAP_ERASE_FAILUIRE         = (100005, "IAP_EraseFailuire", "IAP: Erase Operation Failed")
+    IAP_COMMAND_NOT_SUPPORTED  = (100006, "IAP_CommandNotSupported", "IAP: Specific Command Not Supported")
+    IAP_MEMORY_ACCESS_DISABLED = (100007, "IAP_MemoryAccessDisabled", "IAP: Memory Access Disabled")
 # fmt: on
 
 
@@ -315,5 +348,5 @@ def stringify_status_code(status_code: int) -> str:
     """Stringifies the MBoot status code."""
     return (
         f"{status_code} ({status_code:#x}) "
-        f"{StatusCode.desc(status_code, f'Unknown error code ({status_code})')}."
+        f"{StatusCode.get_description(status_code) if status_code in StatusCode.tags() else f'Unknown error code ({status_code})'}."
     )
