@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """SB31 module of sbfile."""
@@ -41,4 +41,6 @@ __all__ = [
     "CmdSectionHeader",
     # constants
     "EnumCmdTag",
+    # methods
+    "parse_command",
 ]
